import re

import numpy as np
import scipy
from scipy import optimize
from algorithm.parameters import params
from utilities.fitness.math_functions import *
from utilities.fitness.Objective_value import *


def optimize_constants(x, y, ind):
    """
    Use gradient descent to search for values for the constants in
    ind.phenotype which minimise loss.

    :param x: Input (an array of x values).
    :param y: Expected output (expected y values for given inputs).
    :param ind: A GE individual.
    :return: The value of the error metric at those values
    """

    # Save the old phenotype (has c[0] etc, but may not be consecutive from
    # zero)
    ind.phenotype_original = ind.phenotype
    s1 = ind.phenotype.split(',', 1)[0]
    s2 = ind.phenotype.split(',', 1)[1]
    # Parse the phenotype to make the constants consecutive.
    s1, n_consts1 = make_consts_consecutive(s1)
    s2, n_consts2 = make_consts_consecutive(s2)
    s, n_consts = make_consts_consecutive(ind.phenotype)
    # s = ind.phenotype
    # n_consts = 2
    nr_phase = 4
    n_max = 8
    # Create new consecutive constant attribute for individual.
    ind.phenotype_consec_consts = s
    # s1 = s.split(',', 1)[0]
    # s2 = s.split(',', 1)[1]
    # Eval the phenotype.
    # f1 = eval("lambda x: " + s1)
    # f2 = eval("lambda x: " + s2)
    # f = eval("lambda x,c: " + s)
    # Pre-load the error metric fitness function.
    loss = params['ERROR_METRIC']
    function_value_total = np.zeros(int(np.size(y)))
    shape_mismatch_txt = """Shape mismatch between y and yhat. Please check
that your grammar uses the `x[:, 0]` style, not `x[0]`. Please see change
at https://github.com/PonyGE/PonyGE2/issues/130."""

    if n_consts1 == 0 and n_consts2 == 0:
        # ind doesn't refer to c: no need to optimize; just give the immediate objective value
        c = np.zeros(1)
        # yhat = y_hat(c,x,y,f1,f2)
        for i in range(int(np.size(y))):
            y_ind = y[i]
            x_ind = x[:, i]
            yhat = Objective_value_(x_ind, c, s1, s2, n_consts1, n_consts2)
            if not np.isscalar(yhat):
                if y_ind.shape != yhat.shape:
                    raise ValueError(shape_mismatch_txt)
            fval_individual = yhat
            function_value_total[i] = fval_individual
        # ind.opt_consts = c
        assert np.isrealobj(np.sum(function_value_total))
        return np.sum(function_value_total)
    # elif n_consts1 == 0 and n_consts2 != 0:
    #
    #     assert np.isrealobj(np.sum(function_value_total))
    #     return np.sum(function_value_total)
    # elif n_consts1 != 0 and n_consts2 == 0:
    #
    #     assert np.isrealobj(np.sum(function_value_total))
    #     return np.sum(function_value_total)


    for i in range(int(np.size(y))):
        y_ind = y[i]
        x_ind = x[:, i]
        # y_green = y_score(x_ind, y_ind, f)
        obj = lambda c: Objective_value_(x_ind, c, s1, s2, n_consts1, n_consts2)
        theta_consts = n_consts1*2 + n_consts2*4

        # obj = lambda c: loss(y_ind, y_score_4(x_ind, f, c))
    # obj = lambda c: loss(y, y_hat(c,x,y,f)) # we can put c here, outside the grammar ,
    # obj is now a function of c only for L-BFGS-B. Using 0 as the init seems a
    # reasonable choice. But for scipy.curve_fit we might use [1.0] * n_consts.
    # Maybe other minimizers do better with some other choices? There are other
    # methods to try out.
        fval = 100000 * np.ones(n_max)
        for j in range(n_max):
            init = 100 * (np.random.random(theta_consts) - 0.5 * np.ones(theta_consts))
            # init = 100 * (np.random.random(n_consts) - 0.5 * np.ones(n_consts))
            try:
                opti_result = scipy.optimize.minimize(obj, init, method="Powell", options={'disp': False, 'ftol':1e1, 'xtol':1e1}) # local optimization
                # opti_result = scipy.optimize.basinhopping(obj, init)   # global optimization
                fval[j] = opti_result['fun']
            except ValueError:
                raise ValueError("Error during optimization of constants. " \
                                    "Possible cause: " + shape_mismatch_txt)
        fval_individual = np.min(fval)  # for multi-start option
        # fval_individual = fval  # for single-start option
        function_value_total[i] = fval_individual
    # the result is accessed like a dict
    # ind.opt_consts = res['x']  # the optimum values of the constants

    # the most useful form of the phenotype: c[0], c[1] etc replaced
    # with actual numbers, so can be eval'd directly
    # ind.phenotype = replace_consts_with_values(s, ind.opt_consts)
    assert np.isrealobj(np.sum(function_value_total))
    return np.sum(function_value_total)

# def y_score_4(x_ind,f,c):
#     nr_phase = 4
#     x1=np.zeros(nr_phase)
#     x2=np.zeros(nr_phase)
#     x3=np.zeros(nr_phase)
#     x4=np.zeros(nr_phase)
#     for i in range(nr_phase):
#         x1[i] = x_ind[i*nr_phase]
#         x2[i] = x_ind[i*nr_phase+1]
#         x3[i] = x_ind[i*nr_phase+2]
#         x4[i] = x_ind[i*nr_phase+3]
#     y1 = lambda c: f(x1, c)+1
#     y2 = lambda c: f(x2, c)+1
#     y3 = lambda c: f(x3, c)+1
#     y4 = lambda c: f(x4, c)+1
#     y_sum = y1(c) + y2(c) + y3(c) + y4(c)
#     y_green=np.zeros(nr_phase)
#     y_green[0] = 52 * y1(c) / y_sum
#     y_green[1] = 52 * y2(c) / y_sum
#     y_green[2] = 52 * y3(c) / y_sum
#     y_green[3] = 52 * y4(c) / y_sum
#     return y_green
# def y_score_3(x_ind,f,c):
#     nr_phase = 3
#     nr_state = 4
#     x1=np.zeros(nr_state)
#     x2=np.zeros(nr_state)
#     x3=np.zeros(nr_state)
#     for i in range(nr_state):
#         x1[i] = x_ind[i*nr_phase]
#         x2[i] = x_ind[i*nr_phase+1]
#         x3[i] = x_ind[i*nr_phase+2]
#     y1 = lambda c: f(x1, c)+1
#     y2 = lambda c: f(x2, c)+1
#     y3 = lambda c: f(x3, c)+1
#     y_sum = y1(c) + y2(c) + y3(c)
#     y_green=np.zeros(nr_phase)
#     y_green[0] = 54 * y1(c) / y_sum
#     y_green[1] = 54 * y2(c) / y_sum
#     y_green[2] = 54 * y3(c) / y_sum
#     return y_green
# def y_hat(c,x,y,f1,f2):
#     nr_phase = 4
#     y_hat = np.zeros([int(np.size(y)), 1])
#     for i in range(int(np.size(y) / nr_phase)):
#         x_ind = x[:, i * nr_phase]
#         score_q = np.zeros([nr_phase, 1])
#         score_a = np.zeros([nr_phase, 1])
#         for j in range(nr_phase):
#             score_q[j] = f1(x_ind[j])
#             score_a[j] = f2(x_ind[j+nr_phase])
#         score_a_ave=np.average(score_a)
#         score_q_ave=np.average(score_q)
#         for j in range(nr_phase):
#             y_hat[i*nr_phase + j] = 13 + c[0]*(score_q[j] - score_q_ave) + c[1]*(score_a[j] - score_a_ave)
#     return y_hat



def make_consts_consecutive(s):
    """
    The given phenotype will have zero or more occurrences of each const c[0],
    c[1], etc. But eg it might have c[7], but no c[0]. We need to remap, eg:
        7 -> 0
        9 -> 1
    so that we just have c[0], c[1], etc.

    :param s: A given phenotype string.
    :return: The phenotype string but with consecutive constants.
    """

    p = r"c\[(\d+)\]"
    # find the consts, extract idxs as ints, unique-ify and sort
    const_idxs = sorted(map(int, set(re.findall(p, s))))

    for i, j in enumerate(const_idxs):
        ci = "c[%d]" % i
        cj = "c[%d]" % j
        s = s.replace(cj, ci)

    return s, len(const_idxs)


def replace_consts_with_values(s, c):
    """
    Replace the constants in a given string s with the values in a list c.

    :param s: A given phenotype string.
    :param c: A list of values which will replace the constants in the
    phenotype string.
    :return: The phenotype string with the constants replaced.
    """

    for i in range(len(c)):
        s = s.replace("c[%d]" % i, str(c[i]))

    return s