from math import ceil
import numpy as np
np.seterr(all="raise")

def check_relative(x):
    """
    Check if the current relative queue at the intersection is greater or less
    than zero. Returns 1 if greater than zero, returns -1 if less than zero
    """
    
    if x < 0:
        return -1
    else:
        return 1
    
    
def check_order_max(x_list, p):
    """
    Check the order of the maximum queues of the phases at the intersection.
    Return the index of the phase in ASCENDING order. The highest queue gets
    the highest value in return.
    """
    
    queue_temp = x_list[p]
    sort_asc = np.sort(x_list)
    order = np.argwhere(sort_asc == queue_temp)
    return order[0]
    
def check_order_queue(x_list, p):
    """
    Check the order of the queues of the phases at the intersection. 
    Return the index of the phase in ASCENDING order. The highest queue gets
    the highest value in return.
    """
    
    queue_temp = x_list[p]
    sort_asc = np.sort(x_list)
    order = np.argwhere(sort_asc == queue_temp)
    return order[0]   

def check_saturated(x):
    """
    Check if the current phase is saturated by checking if the queue
    can be dissolved in the mean green time. Return -1 if the queue 
    CAN be dissolved. saturated flow rate is 0.83, mean green time = 12.
    """
    
    if x > 0.83*12:
        return 1
    else:
        return -1
